@@ -2,7 +2,6 @@
 Functions to support the retreat of injured units
 ]=]
 
-local H = wesnoth.require "helper"
 local AH = wesnoth.require "ai/lua/ai_helper.lua"
 local BC = wesnoth.require "ai/lua/battle_calcs.lua"
 local LS = wesnoth.require "location_set"
@@ -24,10 +23,12 @@ function retreat_functions.min_hp(unit)
     local retreat_factor = ai.aspects.retreat_factor
 
     -- Leaders are more valuable and should retreat earlier
-    if unit.canrecruit then retreat_factor = retreat_factor * 1.5 end
+    if unit.canrecruit then
+        retreat_factor = retreat_factor * 1.5
+    end
 
     -- Higher retreat willingness on bad terrain
-    local retreat_factor = retreat_factor * (100 - unit:defense_on(wesnoth.current.map[unit])) / 50
+    retreat_factor = retreat_factor * (100 - unit:defense_on(wesnoth.current.map[unit])) / 50
 
     local min_hp = retreat_factor * unit.max_hitpoints
 
@@ -63,9 +64,9 @@ function retreat_functions.retreat_injured_units(units, avoid_map)
                 local abilities = wml.get_child(u.__cfg, "abilities")
                 local regen_amount = 0
                 if abilities then
-                    for regen in wml.child_range(abilities, "regenerate") do
-                        if regen.value > regen_amount then
-                            regen_amount = regen.value
+                    for regenerates in wml.child_range(abilities, "regenerate") do
+                        if (tonumber(regenerates.value) or 0) > regen_amount then
+                            regen_amount = tonumber(regenerates.value)
                         end
                     end
                 end
@@ -113,7 +114,7 @@ function retreat_functions.get_healing_locations(possible_healers)
             local cure = 0
             local abilities = wml.get_child(u.__cfg, "abilities") or {}
             for ability in wml.child_range(abilities, "heals") do
-                heal_amount = ability.value or 0
+                heal_amount = (tonumber(ability.value) or 0)
                 if ability.poison == "slowed" then
                     cure = 1
                 elseif ability.poison == "cured" then
@@ -121,7 +122,7 @@ function retreat_functions.get_healing_locations(possible_healers)
                 end
             end
             if heal_amount + cure > 0 then
-                for x, y in H.adjacent_tiles(u.x, u.y) do
+                for x, y in wesnoth.current.map:iter_adjacent(u) do
                     local old_values = healing_locs:get(x, y) or {0, 0}
                     local best_heal = math.max(old_values[0] or heal_amount)
                     local best_cure = math.max(old_values[1] or cure)
@@ -150,11 +151,10 @@ function retreat_functions.get_retreat_injured_units(healees, regen_amounts, avo
         ally_attack_map = BC.get_attack_map(allies)
     end
 
-    local max_rating, best_loc, best_unit = - math.huge
+    local max_rating, best_loc, best_unit = - math.huge, nil, nil
     for i,u in ipairs(healees) do
+        wesnoth.interface.handle_user_interact()
         local possible_locations = wesnoth.paths.find_reach(u)
-        -- TODO: avoid ally's villages (may be preferable to lower rating so they will
-        -- be used if unit is very injured)
         if (not regen_amounts[i]) then
             -- Unit cannot self heal, make the terrain do it for us if possible
             local location_subset = {}
@@ -168,7 +168,23 @@ function retreat_functions.get_retreat_injured_units(healees, regen_amounts, avo
                     end
                     local curing = 0
                     if heal_amount > 0 then
-                        curing = 2
+                        -- Do not take villages from an allied side
+                        local owner = wesnoth.map.get_owner(loc)
+                        if owner
+                            and (owner ~= wesnoth.current.side)
+                            and (not wesnoth.sides.is_enemy(wesnoth.current.side, owner))
+                        then
+                            -- If allow_ally_villages is true, injured units are allowed to take ally villages.
+                            -- However, they should do so with lower priority, which we do by halving the heal and cure amounts.
+                            if ai.aspects.allow_ally_villages then
+                                heal_amount = heal_amount / 2
+                                curing = 1
+                            else
+                                heal_amount = 0
+                            end
+                        else
+                            curing = 2
+                        end
                     end
                     local healer_values = healing_locs:get(loc[1], loc[2]) or {0, 0}
                     heal_amount = math.max(heal_amount, healer_values[1])
