@@ -1,5 +1,4 @@
 local on_event = wesnoth.require("on_event")
-local helper = wesnoth.require("helper")
 
 local _ = wesnoth.textdomain 'wesnoth-wc'
 
@@ -91,32 +90,40 @@ function artifacts.give_item(unit, index, visualize)
 
 	local object = {
 		wc2_atrifact_id = index,
-		-- cannot filter on wc2_artifact_id being empty so we also need wc2_is_artifact
+		-- cannot filter on wc2_atrifact_id beeing empty so we also need wc2_is_artifact
 		wc2_is_artifact = true,
 	}
 	if make_holder_loyal then
 		table.insert(object, wml.tag.effect { apply_to= "loyal" })
 	end
-		
-		
+
+
 	-- IDEA: i _could_ replace the follwing with a 'apply_to=wc2_artifact' effect that
 	--       basicially applies all effects in the [artifact]s definition. The obvious
-	--       advantage would be a smaller savefile size. Also this would change how savefiles
-	--       would behave if an artifacts effect has changed, i am currently not sure
+	--       advantage would be a smaller savefile size. Also this woudl change how savefiles
+	--       would behve if an artifacts effect has changed, i am currently not sure
 	--       whether that'd be good or bad
 	--
 	--       One of the reasons why i currently won't do this is to make the artifacts list
-	--       more flexible: the suggested approach requires that artifacts are loaded before
+	--       more flixible: the suggested approach requires that artifacts are loaded before
 	--       units are created which means artifacts must be loaded at toplevel [lua] tags
 	for i, effect in ipairs(aftifact_data.effect) do
 		table.insert(object, wml.tag.effect (effect) )
 	end
-	local unit_initial_hp = unit.hitpoints
 	unit:add_modification("object", object)
+	local unit_initial_hp = unit.hitpoints
 	--rebuild unit, to reduce savefile size.
 	unit:transform(unit.type)
 	-- restore unit hitpoints to before they picked up the artifact
 	unit.hitpoints = unit_initial_hp
+
+	-- slot in traits if any artifacts grant any
+	for trait in wml.child_range(artifacts.list[index], "trait") do
+		if not unit:matches { wml.tag.filter_wml { wml.tag.modifications { wml.tag.trait { id = trait.id } } } } then
+			unit:add_modification("trait", trait)
+		end
+	end
+
 	-- the artifact might reduce the max xp.
 	unit:advance(true, true)
 end
@@ -129,7 +136,7 @@ on_event("wc2_drop_pickup", function(ec)
 		return
 	end
 
-	if not unit then 
+	if not unit then
 		return
 	end
 
@@ -138,7 +145,7 @@ on_event("wc2_drop_pickup", function(ec)
 	if not wml.variables["wc2_config_experimental_pickup"] and not is_human  then
 		return
 	end
-	
+
 
 	local index = item.variables.wc2_atrifact_id
 	local filter = artifacts.get_artifact(index).filter
@@ -157,7 +164,7 @@ on_event("wc2_drop_pickup", function(ec)
 			return
 		end
 	end
-	
+
 
 	wc2_dropping.item_taken = true
 	artifacts.give_item(unit, index, true)
@@ -166,7 +173,7 @@ end)
 
 -- returns a list of artifact ids, suitable for  the give type ('enemy' for example).
 function artifacts.fresh_artifacts_list(for_type)
-	local res = {} 
+	local res = {}
 	for i,v in ipairs(artifacts.get_artifact_list()) do
 		if not for_type or not stringx.map_split(v.not_available or "")[for_type] then
 			table.insert(res, i)
@@ -198,7 +205,7 @@ on_event("die", function(event_context)
 end)
 
 -- returns true if there is an item in the map at the given position,
--- used to determine whether to show the artifact info menu at that position. 
+-- used to determine whether to show the artifact info menu at that position.
 function artifacts.is_item_at(x,y)
 	for i,item in ipairs(wesnoth.interface.get_items(x,y)) do
 		if item.variables.wc2_atrifact_id then
@@ -208,7 +215,7 @@ function artifacts.is_item_at(x,y)
 	return false
 end
 
--- shows an information [message] about the item laying at position 
+-- shows an information [message] about the item laying at position
 -- @a cfg.x, cfg.y
 function wesnoth.wml_actions.wc2_show_item_info(cfg)
 	local x = cfg.x
